"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from clerk_backend_api.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional, TypedDict
from typing_extensions import NotRequired


class CreateUserPublicMetadataTypedDict(TypedDict):
    r"""Metadata saved on the user, that is visible to both your Frontend and Backend APIs"""
    
    

class CreateUserPublicMetadata(BaseModel):
    r"""Metadata saved on the user, that is visible to both your Frontend and Backend APIs"""
    
    

class CreateUserPrivateMetadataTypedDict(TypedDict):
    r"""Metadata saved on the user, that is only visible to your Backend API"""
    
    

class CreateUserPrivateMetadata(BaseModel):
    r"""Metadata saved on the user, that is only visible to your Backend API"""
    
    

class CreateUserUnsafeMetadataTypedDict(TypedDict):
    r"""Metadata saved on the user, that can be updated from both the Frontend and Backend APIs.
    Note: Since this data can be modified from the frontend, it is not guaranteed to be safe.
    """
    
    

class CreateUserUnsafeMetadata(BaseModel):
    r"""Metadata saved on the user, that can be updated from both the Frontend and Backend APIs.
    Note: Since this data can be modified from the frontend, it is not guaranteed to be safe.
    """
    
    

class CreateUserRequestBodyTypedDict(TypedDict):
    external_id: NotRequired[Nullable[str]]
    r"""The ID of the user as used in your external systems or your previous authentication solution.
    Must be unique across your instance.
    """
    first_name: NotRequired[Nullable[str]]
    r"""The first name to assign to the user"""
    last_name: NotRequired[Nullable[str]]
    r"""The last name to assign to the user"""
    email_address: NotRequired[List[str]]
    r"""Email addresses to add to the user.
    Must be unique across your instance.
    The first email address will be set as the user's primary email address.
    """
    phone_number: NotRequired[List[str]]
    r"""Phone numbers to add to the user.
    Must be unique across your instance.
    The first phone number will be set as the user's primary phone number.
    """
    web3_wallet: NotRequired[List[str]]
    r"""Web3 wallets to add to the user.
    Must be unique across your instance.
    The first wallet will be set as the user's primary wallet.
    """
    username: NotRequired[Nullable[str]]
    r"""The username to give to the user.
    It must be unique across your instance.
    """
    password: NotRequired[Nullable[str]]
    r"""The plaintext password to give the user.
    Must be at least 8 characters long, and can not be in any list of hacked passwords.
    """
    password_digest: NotRequired[str]
    r"""In case you already have the password digests and not the passwords, you can use them for the newly created user via this property.
    The digests should be generated with one of the supported algorithms.
    The hashing algorithm can be specified using the `password_hasher` property.
    """
    password_hasher: NotRequired[str]
    r"""The hashing algorithm that was used to generate the password digest.

    The algorithms we support at the moment are [`bcrypt`](https://en.wikipedia.org/wiki/Bcrypt), [`bcrypt_sha256_django`](https://docs.djangoproject.com/en/4.0/topics/auth/passwords/), [`md5`](https://en.wikipedia.org/wiki/MD5), `pbkdf2_sha1`, `pbkdf2_sha256`, [`pbkdf2_sha256_django`](https://docs.djangoproject.com/en/4.0/topics/auth/passwords/),
    [`phpass`](https://www.openwall.com/phpass/), [`scrypt_firebase`](https://firebaseopensource.com/projects/firebase/scrypt/),
    [`scrypt_werkzeug`](https://werkzeug.palletsprojects.com/en/3.0.x/utils/#werkzeug.security.generate_password_hash), [`sha256`](https://en.wikipedia.org/wiki/SHA-2),
    and the [`argon2`](https://argon2.online/) variants: `argon2i` and `argon2id`.

    Each of the supported hashers expects the incoming digest to be in a particular format. See the [Clerk docs](https://clerk.com/docs/references/backend/user/create-user) for more information.
    """
    skip_password_checks: NotRequired[bool]
    r"""When set to `true` all password checks are skipped.
    It is recommended to use this method only when migrating plaintext passwords to Clerk.
    Upon migration the user base should be prompted to pick stronger password.
    """
    skip_password_requirement: NotRequired[bool]
    r"""When set to `true`, `password` is not required anymore when creating the user and can be omitted.
    This is useful when you are trying to create a user that doesn't have a password, in an instance that is using passwords.
    Please note that you cannot use this flag if password is the only way for a user to sign into your instance.
    """
    totp_secret: NotRequired[str]
    r"""In case TOTP is configured on the instance, you can provide the secret to enable it on the newly created user without the need to reset it.
    Please note that currently the supported options are:
    * Period: 30 seconds
    * Code length: 6 digits
    * Algorithm: SHA1
    """
    backup_codes: NotRequired[List[str]]
    r"""If Backup Codes are configured on the instance, you can provide them to enable it on the newly created user without the need to reset them.
    You must provide the backup codes in plain format or the corresponding bcrypt digest.
    """
    public_metadata: NotRequired[CreateUserPublicMetadataTypedDict]
    r"""Metadata saved on the user, that is visible to both your Frontend and Backend APIs"""
    private_metadata: NotRequired[CreateUserPrivateMetadataTypedDict]
    r"""Metadata saved on the user, that is only visible to your Backend API"""
    unsafe_metadata: NotRequired[CreateUserUnsafeMetadataTypedDict]
    r"""Metadata saved on the user, that can be updated from both the Frontend and Backend APIs.
    Note: Since this data can be modified from the frontend, it is not guaranteed to be safe.
    """
    delete_self_enabled: NotRequired[Nullable[bool]]
    r"""If enabled, user can delete themselves via FAPI.

    """
    legal_accepted_at: NotRequired[Nullable[str]]
    r"""A custom timestamp denoting _when_ the user accepted legal requirements, specified in RFC3339 format (e.g. `2012-10-20T07:15:20.902Z`)."""
    skip_legal_checks: NotRequired[Nullable[bool]]
    r"""When set to `true` all legal checks are skipped.
    It is not recommended to skip legal checks unless you are migrating a user to Clerk.
    """
    create_organization_enabled: NotRequired[Nullable[bool]]
    r"""If enabled, user can create organizations via FAPI.

    """
    create_organizations_limit: NotRequired[Nullable[int]]
    r"""The maximum number of organizations the user can create. 0 means unlimited.

    """
    created_at: NotRequired[str]
    r"""A custom date/time denoting _when_ the user signed up to the application, specified in RFC3339 format (e.g. `2012-10-20T07:15:20.902Z`)."""
    

class CreateUserRequestBody(BaseModel):
    external_id: OptionalNullable[str] = UNSET
    r"""The ID of the user as used in your external systems or your previous authentication solution.
    Must be unique across your instance.
    """
    first_name: OptionalNullable[str] = UNSET
    r"""The first name to assign to the user"""
    last_name: OptionalNullable[str] = UNSET
    r"""The last name to assign to the user"""
    email_address: Optional[List[str]] = None
    r"""Email addresses to add to the user.
    Must be unique across your instance.
    The first email address will be set as the user's primary email address.
    """
    phone_number: Optional[List[str]] = None
    r"""Phone numbers to add to the user.
    Must be unique across your instance.
    The first phone number will be set as the user's primary phone number.
    """
    web3_wallet: Optional[List[str]] = None
    r"""Web3 wallets to add to the user.
    Must be unique across your instance.
    The first wallet will be set as the user's primary wallet.
    """
    username: OptionalNullable[str] = UNSET
    r"""The username to give to the user.
    It must be unique across your instance.
    """
    password: OptionalNullable[str] = UNSET
    r"""The plaintext password to give the user.
    Must be at least 8 characters long, and can not be in any list of hacked passwords.
    """
    password_digest: Optional[str] = None
    r"""In case you already have the password digests and not the passwords, you can use them for the newly created user via this property.
    The digests should be generated with one of the supported algorithms.
    The hashing algorithm can be specified using the `password_hasher` property.
    """
    password_hasher: Optional[str] = None
    r"""The hashing algorithm that was used to generate the password digest.

    The algorithms we support at the moment are [`bcrypt`](https://en.wikipedia.org/wiki/Bcrypt), [`bcrypt_sha256_django`](https://docs.djangoproject.com/en/4.0/topics/auth/passwords/), [`md5`](https://en.wikipedia.org/wiki/MD5), `pbkdf2_sha1`, `pbkdf2_sha256`, [`pbkdf2_sha256_django`](https://docs.djangoproject.com/en/4.0/topics/auth/passwords/),
    [`phpass`](https://www.openwall.com/phpass/), [`scrypt_firebase`](https://firebaseopensource.com/projects/firebase/scrypt/),
    [`scrypt_werkzeug`](https://werkzeug.palletsprojects.com/en/3.0.x/utils/#werkzeug.security.generate_password_hash), [`sha256`](https://en.wikipedia.org/wiki/SHA-2),
    and the [`argon2`](https://argon2.online/) variants: `argon2i` and `argon2id`.

    Each of the supported hashers expects the incoming digest to be in a particular format. See the [Clerk docs](https://clerk.com/docs/references/backend/user/create-user) for more information.
    """
    skip_password_checks: Optional[bool] = None
    r"""When set to `true` all password checks are skipped.
    It is recommended to use this method only when migrating plaintext passwords to Clerk.
    Upon migration the user base should be prompted to pick stronger password.
    """
    skip_password_requirement: Optional[bool] = None
    r"""When set to `true`, `password` is not required anymore when creating the user and can be omitted.
    This is useful when you are trying to create a user that doesn't have a password, in an instance that is using passwords.
    Please note that you cannot use this flag if password is the only way for a user to sign into your instance.
    """
    totp_secret: Optional[str] = None
    r"""In case TOTP is configured on the instance, you can provide the secret to enable it on the newly created user without the need to reset it.
    Please note that currently the supported options are:
    * Period: 30 seconds
    * Code length: 6 digits
    * Algorithm: SHA1
    """
    backup_codes: Optional[List[str]] = None
    r"""If Backup Codes are configured on the instance, you can provide them to enable it on the newly created user without the need to reset them.
    You must provide the backup codes in plain format or the corresponding bcrypt digest.
    """
    public_metadata: Optional[CreateUserPublicMetadata] = None
    r"""Metadata saved on the user, that is visible to both your Frontend and Backend APIs"""
    private_metadata: Optional[CreateUserPrivateMetadata] = None
    r"""Metadata saved on the user, that is only visible to your Backend API"""
    unsafe_metadata: Optional[CreateUserUnsafeMetadata] = None
    r"""Metadata saved on the user, that can be updated from both the Frontend and Backend APIs.
    Note: Since this data can be modified from the frontend, it is not guaranteed to be safe.
    """
    delete_self_enabled: OptionalNullable[bool] = UNSET
    r"""If enabled, user can delete themselves via FAPI.

    """
    legal_accepted_at: OptionalNullable[str] = UNSET
    r"""A custom timestamp denoting _when_ the user accepted legal requirements, specified in RFC3339 format (e.g. `2012-10-20T07:15:20.902Z`)."""
    skip_legal_checks: OptionalNullable[bool] = UNSET
    r"""When set to `true` all legal checks are skipped.
    It is not recommended to skip legal checks unless you are migrating a user to Clerk.
    """
    create_organization_enabled: OptionalNullable[bool] = UNSET
    r"""If enabled, user can create organizations via FAPI.

    """
    create_organizations_limit: OptionalNullable[int] = UNSET
    r"""The maximum number of organizations the user can create. 0 means unlimited.

    """
    created_at: Optional[str] = None
    r"""A custom date/time denoting _when_ the user signed up to the application, specified in RFC3339 format (e.g. `2012-10-20T07:15:20.902Z`)."""
    
    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["external_id", "first_name", "last_name", "email_address", "phone_number", "web3_wallet", "username", "password", "password_digest", "password_hasher", "skip_password_checks", "skip_password_requirement", "totp_secret", "backup_codes", "public_metadata", "private_metadata", "unsafe_metadata", "delete_self_enabled", "legal_accepted_at", "skip_legal_checks", "create_organization_enabled", "create_organizations_limit", "created_at"]
        nullable_fields = ["external_id", "first_name", "last_name", "username", "password", "delete_self_enabled", "legal_accepted_at", "skip_legal_checks", "create_organization_enabled", "create_organizations_limit"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (self.__pydantic_fields_set__.intersection({n}) or k in null_default_fields) # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
        
